#!/usr/bin/env python3
"""
The **yace** Interface Model (``yim``) is defined by plain-text files in YAML
format, these are described

* Yace Interface Model files (YAML Format)
* Entities
* Interface

The interface model was initially just a collection of YAML-files, globbed for
and merged together by the copiler using a YAML-parser. This left for two
things to be done:

* Documentation of the interface model
* Validation of the interface model

"""
